package be.ulb.db.index.btree;

import java.nio.ByteBuffer;

import be.ulb.db.Block;
import be.ulb.db.BlockAddress;
import be.ulb.db.BlockFactory;
import be.ulb.db.schema.Schema;

public class BTIndexBlockFactory extends BlockFactory
{
    public enum BlockInstanciationMode {
        /**
         * Only reload data from disk.
         */
        FromRecord,

        /**
         * Only create leaves.
         */
        Leaf,

        /**
         * Only create inner blocks.
         */
        Inner
    }

    Schema recordSchema;
    BlockInstanciationMode instantiation;

    /**
     * Construct a B+Tree index blocks factory. The index stores keys of the
     * specified [[recordSchema]]. Depending on the [[instantiation]] this
     * factory generated leaves, inner blocks, or only reload blocks from
     * existing blocks.
     */
    public BTIndexBlockFactory(Schema recordSchema, BlockInstanciationMode instantiation)
    {
        this.recordSchema = recordSchema;
        this.instantiation = instantiation;
    }

    /**
     * Create a new B+Tree with the factory schema and instantiation mode.
     */
    public Block create(ByteBuffer buff, BlockAddress addr) {
        BlockInstanciationMode instantiation = this.instantiation;

        //XXX: The factories should know whether they are reviving an old block or
        //creating a new one.
        if (instantiation == BlockInstanciationMode.FromRecord) {
            if (BTIndexBlock.isLeafBlock(buff))
                instantiation = BlockInstanciationMode.Leaf;
            else
                instantiation = BlockInstanciationMode.Inner;
        }

        // Create the right kind of block
        if (instantiation == BlockInstanciationMode.Leaf) {
            return new BTIndexBlockLeaf(recordSchema, buff, addr);
        }
        else if (instantiation == BlockInstanciationMode.Inner) {
            return new BTIndexBlockInner(recordSchema, buff, addr);
        }

        return null; // Should not happen
    }
}
