package be.ulb.db.index.btree;

import java.io.IOException;
import java.util.NoSuchElementException;

import be.ulb.db.RecordAddress;
import be.ulb.db.RecordAddressIterator;
import be.ulb.db.buffermanager.BufferManager;
import be.ulb.db.buffermanager.BufferManagerException;
import be.ulb.db.diskmanager.DiskManagerException;

public class BTIndexIterator implements RecordAddressIterator {
    BufferManager bm;
    BTIndexBlockLeaf currentLeaf;
    int slot;
    boolean eof;

    public void invalidate() {
        if (eof) return;

        try {
        bm.unpinBlock(currentLeaf);
        }
        catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
        eof = true;
    }

    private boolean loadNextSibling() {
        if (currentLeaf.hasNextSibling()) {
            try {
                BTIndexBlockLeaf nextleaf = currentLeaf.getNextSibling(bm);
                bm.unpinBlock(currentLeaf);
                currentLeaf = nextleaf;
            }
            catch (Exception e) {
                e.printStackTrace();
                System.exit(1);
            }

            slot = 0;
            eof = false;
            return true;
        }
        return false;
    }

    private boolean ensureSlotAvailable() {
        return (slot < currentLeaf.size() || loadNextSibling());
    }

    public BTIndexIterator(BufferManager bm, BTIndexBlockLeaf leaf, int slot) {
        this.bm = bm;
        this.slot = slot;
        this.currentLeaf = leaf;
        this.eof = !ensureSlotAvailable();
    }

    /* standard iterator stuff */
    public boolean hasNext() {
        return (!eof);
    }

    public RecordAddress next() throws NoSuchElementException , IOException,
				BufferManagerException, DiskManagerException {
        if (eof) throw new NoSuchElementException();

        RecordAddress result = currentLeaf.getRecordAddressAt(slot);
        this.slot++;
        this.eof = ensureSlotAvailable();
        return result;
    }

    /* closes the attached resources. Calling other methods after a call to close() can fail */
    public void close() {
        invalidate();
    }
}
