/*-------------------------------------------------------------------
   Authors: 2011: Pierre Gewelt

   AttributeTypeByte represent the field type "BYTE" that can occur in a
   schema.

   $Id$
  -------------------------------------------------------------------*/
package be.ulb.db.schema;
import java.nio.ByteBuffer;

public class AttributeTypeByte extends AttributeType
{
	public AttributeTypeByte()
	{
	}

	/* A java object is a valid instance of this field type if it is
	 * a byte */
	public boolean hasInstance(Object o)
	{
		return (o instanceof Byte);
	}

	/* returns the size of a value in bytes */
	public int bytesize()
	{
		return 1; // trivial, one byte fits into one byte
	}

	/* writes an instance of the field type to the byte buffer.  We
	 * assume the bytebuffer's position and limit have been set
	 * correctly to the start end end of this field */
	public void writeInstanceToByteBuffer(ByteBuffer buff, Object instance)
	{
		byte myByte = ((Byte) instance).byteValue();
		buff.put(myByte);
	}

	/* reads an instance of the field type from a byte buffer.  We
	 * assume the bytebuffer's position and limit have been set
	 * correctly to the start end end of this field */
	public Object readInstanceFromByteBuffer(ByteBuffer buff)
	{
		byte b = buff.get();
		return new Byte(b);
	}

	public boolean equals(Object o)
	{
		return (o instanceof AttributeTypeByte);
	}

}
